@@ -28,9 +28,16 @@ function scrollbarWidth() {
 function reloadGraphs(max_image_width) {
     $('a.replace-with-image').each(
         function (index) {
-            var url, url_click, timestamp, width, height, amp_or_questionmark, html_img;
+            var url, url_click, timestamp, width, height, amp_or_questionmark, 
+            html_img, image, $main_tag, html_src, html_url, errormsg;
+            $main_tag = $(this);
             width = $(this).parent('.img-use-my-size').innerWidth();
             height = $(this).parent('.img-use-my-size').innerHeight();
+            if ($(this).attr('data-errormsg')) {
+                errormsg = $(this).attr('data-errormsg');
+            } else {
+                errormsg = 'An error occurred';
+            }
             if (width === null) {
                 width = '';
                 height = '';
@@ -58,16 +65,40 @@ function reloadGraphs(max_image_width) {
             // Remove a previous image that's already there.
             $('~ img', this).remove();
             timestamp = new Date().getTime();  // No cached images.
-            html_img = '<img src="' + url +
+            html_url = url +
                 amp_or_questionmark + 'width=' + width +
                 '&height=' + height +
-                '&random=' + timestamp + '" ' +
+                '&random=' + timestamp;
+            html_img = '<img src="' + html_url + '" class="auto-inserted" ' +
                 '/>';
-            // place <a href></a> around image
+            // Remove progress animation and possibly old images.
+            $main_tag.parent().find(".auto-inserted").remove();
+            // Add progress animation.
+            $(this).after('<div class="auto-inserted"><img src="/static_media/lizard_ui/ajax-loader.gif" class="progress-animation" data-src="' + html_url + '" /></div>');
+
+            // Preload image.
+            image = $(html_img);
+
+            // Place <a href></a> around image tag.
             if (url_click !== undefined) {
-                html_img = '<a href="' + url_click + '">' + html_img + '</a>';
+                html_img = '<a href="' + url_click + '" class="auto-inserted">' + html_img + '</a>';
             }
-            $(this).after(html_img);
+
+            image.load(function () {
+                // After preloading.
+                // Remove progress animation and possibly old images.
+                $main_tag.parent().find(".auto-inserted").remove();
+                $main_tag.after(html_img);
+            });
+            image.error(function () {
+                // After preloading.
+                // Remove progress animation and possibly old images.
+                $main_tag.parent().find(".auto-inserted").remove();
+                $main_tag.after(
+                    '<p class="auto-inserted">' +
+                        errormsg +
+                        '</p>');
+            });
         }
     );
 }
@@ -82,8 +113,9 @@ function printPage() {
     }
     // Make images smaller
     reloadGraphs(max_image_width);
-    setTimeout(window.print, 500);
-    // TODO: add link you can click to restore the normal view.
+    setTimeout(function () {
+        window.print();
+    }, 500);
 }
 
 
@@ -252,6 +284,54 @@ function setUpCollapsibleSidebar() {
 }
 
 
+/* Initializes and shows (legend) tooltip. Must be re-initialized
+after lizard-map workspace update.  */
+function setUpTooltips() {
+    $(".legend-tooltip").each(function () {
+        if (!$(this).data("popup-initialized")) {
+            $(this).data("popup-initialized", true);
+            $(this).tooltip({
+                position: 'center right',
+                effect: 'fade',
+                onShow: function () {
+                    var offset, pixels_below_screen;
+                    // Too high?
+                    offset = this.getTip().offset();
+                    if (offset.top < 0) {
+                        offset.top = 0;
+                        this.getTip().offset(offset);
+                    }
+                    // Too low?
+                    pixels_below_screen = offset.top +
+                        this.getTip().height() -
+                        $(window).height();
+                    if (pixels_below_screen > 0) {
+                        offset.top = offset.top - pixels_below_screen;
+                        this.getTip().offset(offset);
+                    }
+                    // Repositioning beforehand would be visually nicer.
+                }
+            });
+        }
+    });
+}
+
+
+/*
+Sets up treestructures with an interactive interface. Use force_initialize.
+*/
+function setUpTree(force_initialize) {
+    $(".automatic-tree").each(function () {
+        if (!$(this).data("tree-initialized") || (force_initialize)) {
+            $(this).data("tree-initialized", true);
+            $(this).treeview({
+                collapsed: true
+            });
+        }
+    });
+}
+
+
 function setUpAccordion() {
     $("#accordion").tabs(
         "#accordion .pane",
@@ -260,13 +340,24 @@ function setUpAccordion() {
     /* Set up a global 'accordion' variable to later steer the animation. */
     accordion = $("#accordion").tabs();
     $(".accordion-load-next a").live('click', function (event) {
-        var pane, nextPaneId, url;
+        var pane, nextPaneId, url, newTitle;
         event.preventDefault();
         pane = $(this).parents(".accordion-load-next");
         nextPaneId = pane.attr("data-next-pane-id");
         url = $(this).attr("href");
         $(nextPaneId).html('<div class="loading" />');
-        $(nextPaneId).load(url + " " + nextPaneId);
+        $.get(url, {}, function (data) {
+            // Update all panes. Data is the whole (new) html page.
+            $(".pane").each(function () {
+                // Title of current pane.
+                newTitle = $(data).find("#" + $(this).attr("id")).prev().html();
+                $(this).prev().html(newTitle);
+                // Content of current pane.
+                $(this).html($(data).find("#" + $(this).attr("id")).html());
+            });
+            setUpTooltips();
+            setUpTree(true);
+        });
         $("li.selected", pane).removeClass("selected");
         $(this).parent("li").addClass("selected");
         accordion.click(accordion.getIndex() + 1);
@@ -274,18 +365,6 @@ function setUpAccordion() {
 }
 
 
-function setUpTree() {
-    $(".automatic-tree").each(function () {
-        if (!$(this).data("tree-initialized")) {
-            $(this).data("tree-initialized", true);
-            $(this).treeview({
-                collapsed: true
-            });
-        }
-    });
-}
-
-
 /* Turn all objects with class "popup-trigger" into jquery overlay triggers. */
 function setUpOverlays() {
     $(".popup-trigger").each(function () {
@@ -311,15 +390,20 @@ function setUpLogin() {
             url,
             $form.serialize(),
             function (data) {
-                if (data === "true") {
+                if (data.success === true) {
                     // Login successful.
-                    window.location.reload();
+                    if ((data.next === "") || (data.next === null)) {
+                        window.location.reload();
+                    } else {
+                        window.location = data.next;
+                    }
                 } else {
                     // Login unsuccessful.
                     $("#login-form").find(".close").click();
                     $("#wrong-login").overlay({load: true });
                 }
-            });
+            },
+            'json');
         return false;  // Prevents propagation to original form submit.
     });
     $("#logout-button").click(function () {
@@ -328,9 +412,11 @@ function setUpLogin() {
         $.post(
             url,
             {},
-            function (data) {
-                window.location.reload();
-            });
+            function (event) {
+                window.location = "/";
+                return false;
+            }
+        );
     });
 }
 
@@ -347,6 +433,24 @@ function setUpScreen() {
 }
 
 
+function setUpPortalTabs() {
+    var selected_tab;
+    selected_tab = $("#portal-tab-selected").attr("data-selected");
+    if (selected_tab === undefined) {
+        selected_tab = $("#portal-tab-selected-default")
+            .attr("data-selected");
+    }
+    $(selected_tab).addClass("selected");
+}
+
+function restretchExistingElements() {
+    fillScreen();
+    divideVerticalSpaceEqually();
+    stretchOneSidebarBox();
+    reloadGraphs();
+}
+
+
 /* Fill the screen (again) when we open the page and when the window is
    resized.
 */
@@ -356,13 +460,8 @@ $(window).resize(function () {
         clearTimeout(resizeTimer);
     }
     resizeTimer = setTimeout(
-        function () {
-            fillScreen();
-            divideVerticalSpaceEqually();
-            stretchOneSidebarBox();
-            reloadGraphs();
-        },
-        300);
+            restretchExistingElements, 300
+    );
 });
 
 
@@ -379,4 +478,11 @@ $(document).ready(function () {
     setUpOverlays();
     setUpLogin();
     setUpPrintButton();
+
+    //setUpWorkspaceAcceptableButtons();
+    // Light up the selected tab, if available.
+    setUpPortalTabs();
+
+    // Set up legend.
+    setUpTooltips(); // The edit function is on the tooltip.
 });
