@@ -9,6 +9,7 @@ INSTALLED_APPS = [
     'lizard_ui',
     'compressor',
     'staticfiles',
+    'south',
     'django_nose',
     'django.contrib.admin',
     'django.contrib.auth',
@@ -81,5 +82,5 @@ TEMPLATE_CONTEXT_PROCESSORS = (
     "django.core.context_processors.media",
     # Needs to be added for django-staticfiles to allow you
     # to use {{ STATIC_URL }}myapp/my.css in your templates.
-    'staticfiles.context_processors.static_url',
+    #'staticfiles.context_processors.static_url',
     )
